@@ -55,6 +55,7 @@ typedef struct _GogRegEqn	GogRegEqn;
 typedef struct _GogTrendLineType	GogTrendLineType;
 typedef struct _GogSeriesLines GogSeriesLines;
 typedef struct _GogSmoothedCurve	GogSmoothedCurve;
+typedef struct _Gog3DBox	Gog3DBox;
 
 /* formating */
 typedef struct _GogTheme	GogTheme;
@@ -95,6 +96,7 @@ typedef enum {
 	GOG_AXIS_VIRTUAL,
 	GOG_AXIS_PSEUDO_3D = GOG_AXIS_VIRTUAL,
 	GOG_AXIS_COLOR,
+	GOG_AXIS_BUBBLE,
 	GOG_AXIS_TYPES
 } GogAxisType;
 
@@ -105,6 +107,7 @@ typedef enum {
 	GOG_AXIS_SET_XY		  = (1 << GOG_AXIS_X) | (1 << GOG_AXIS_Y),
 	GOG_AXIS_SET_XY_pseudo_3d = (1 << GOG_AXIS_X) | (1 << GOG_AXIS_Y) | (1 << GOG_AXIS_PSEUDO_3D),
 	GOG_AXIS_SET_XY_COLOR 	  = (1 << GOG_AXIS_X) | (1 << GOG_AXIS_Y) | (1 << GOG_AXIS_COLOR),
+	GOG_AXIS_SET_XY_BUBBLE 	  = (1 << GOG_AXIS_X) | (1 << GOG_AXIS_Y) | (1 << GOG_AXIS_BUBBLE),
 	GOG_AXIS_SET_XYZ	  = (1 << GOG_AXIS_X) | (1 << GOG_AXIS_Y) | (1 << GOG_AXIS_Z),
 	GOG_AXIS_SET_RADAR        = (1 << GOG_AXIS_CIRCULAR) | (1 << GOG_AXIS_RADIAL),
 	GOG_AXIS_SET_FUNDAMENTAL  = ((1 << GOG_AXIS_VIRTUAL) - 1),
