@@ -21,6 +21,9 @@
 
 #include <goffice/goffice-config.h>
 #include <goffice/graph/gog-chart-impl.h>
+#include <goffice/graph/gog-chart-map.h>
+#include <goffice/graph/gog-chart-map-3d.h>
+#include <goffice/graph/gog-3d-box.h>
 #include <goffice/graph/gog-plot-impl.h>
 #include <goffice/graph/gog-graph-impl.h>
 #include <goffice/graph/gog-style.h>
@@ -30,7 +33,8 @@
 #include <goffice/graph/gog-grid.h>
 #include <goffice/graph/gog-grid-line.h>
 #include <goffice/graph/gog-renderer.h>
-#include <goffice/utils/go-math.h>
+#include <goffice/math/go-math.h>
+#include <goffice/math/go-matrix3x3.h>
 
 #include <gsf/gsf-impl-utils.h>
 #include <glib/gi18n-lib.h>
@@ -79,348 +83,6 @@ gog_axis_set_from_str (char const *str)
 }
 
 /*****************************************************************************/
-
-struct _GogChartMap {
-	GogChart 		*chart;
-	GogViewAllocation	 area;
-	gpointer	 	 data;
-	GogAxisMap		*axis_map[3];
-	gboolean		 is_valid;
-
-	void (*map_2D_to_view)	(GogChartMap *map, double x, double y, double *u, double *v);
-};
-
-static void
-calc_polygon_parameters (GogViewAllocation const *area, GogChartMapPolarData *data, gboolean fill_area)
-{
-	double edges = data->th1 - data->th0 + 1.0;
-	double width = 2.0 * sin (2.0 * M_PI * go_rint (edges / 4.0) / edges);
-	double height = 1.0 - cos (2.0 * M_PI * go_rint (edges / 2.0) / edges);
-
-	data->rx = area->w / width;
-	data->ry = area->h / height;
-
-	if (!fill_area) {
-		data->rx = MIN (data->rx, data->ry);
-		data->ry = data->rx;
-	}
-
-	data->cx = area->x + area->w / 2.0;
-	data->cy = area->y + data->ry + (area->h - data->ry * height) / 2.0;
-}
-
-static void
-calc_circle_parameters (GogViewAllocation const *area, GogChartMapPolarData *data, gboolean fill_area)
-{
-	double x_min, x_max, y_min, y_max;
-	
-	if (data->th0 >= data->th1) {
-		x_min = y_min = -1.0;
-		x_max = y_max = 1.0;
-	} else {
-		double x;
-
-		if (data->th0 < -2.0 * M_PI) {
-			x = data->th0 - fmod (data->th0, 2.0 * M_PI);
-			data->th0 -= x;
-			data->th1 -= x;
-		} else if (data->th1 > 2.0 * M_PI) {
-			x = data->th1 - fmod (data->th1, 2.0 * M_PI);
-			data->th0 += x;
-			data->th1 += x;
-		}
-		if (data->th1 - data->th0 > go_add_epsilon (2 * M_PI)) 
-			data->th1 = data->th0 + 2 * M_PI; 
-
-		x_min = x_max = y_min = y_max = 0;
-		x = cos (-data->th0);
-		x_min = MIN (x_min, x); x_max = MAX (x_max, x);
-		x = sin (-data->th0);
-		y_min = MIN (y_min, x); y_max = MAX (y_max, x);
-		x = cos (-data->th1);
-		x_min = MIN (x_min, x); x_max = MAX (x_max, x);
-		x = sin (-data->th1);
-		y_min = MIN (y_min, x); y_max = MAX (y_max, x);
-
-		if (0 > data->th0 && 0 < data->th1)
-			x_max = 1.0;
-		if ((       M_PI / 2.0 > data->th0 &&        M_PI / 2.0 < data->th1) ||
-		    (-3.0 * M_PI / 2.0 > data->th0 && -3.0 * M_PI / 2.0 < data->th1))
-			y_min = -1.0;
-		if (( M_PI > data->th0 &&  M_PI < data->th1) ||
-		    (-M_PI > data->th0 && -M_PI < data->th1))
-			x_min = -1.0;
-		if (( 3.0 * M_PI / 2.0 > data->th0 &&  3.0 * M_PI / 2.0 < data->th1) ||
-		    (-      M_PI / 2.0 > data->th0 && -      M_PI / 2.0 < data->th1))
-			y_max = 1.0;
-	}
-	data->rx = area->w / (x_max - x_min);
-	data->ry = area->h / (y_max - y_min);
-	if (!fill_area) {
-		data->rx = MIN (data->rx, data->ry);
-		data->ry = data->rx;
-	}	
-	data->cx = -x_min * data->rx + area->x + (area->w - data->rx * (x_max - x_min)) / 2.0;
-	data->cy = -y_min * data->ry + area->y + (area->h - data->ry * (y_max - y_min)) / 2.0;
-}
-
-static void 
-null_map_2D (GogChartMap *map, double x, double y, double *u, double *v)
-{
-	g_warning ("[GogChartMap::map_2D] not implemented");
-}
-
-typedef struct {
-	double a, b;
-} XMapData;
-
-static void 
-x_map_2D_to_view (GogChartMap *map, double x, double y, double *u, double *v)
-{
-	XMapData *data = map->data;
-
-	*u = gog_axis_map_to_view (map->axis_map[0], x);
-	*v = data->a * y + data->b;
-}
-
-typedef struct {
-	double		a[2][2];
-	double		b[2];
-} XYMapData;
-
-static void
-xy_map_2D_to_view (GogChartMap *map, double x, double y, double *u, double *v)
-{
-	*u = gog_axis_map_to_view (map->axis_map[0], x);
-	*v = gog_axis_map_to_view (map->axis_map[1], y);
-}
-
-static void
-polar_map_2D_to_view (GogChartMap *map, double x, double y, double *u, double *v)
-{
-	GogChartMapPolarData *data = (GogChartMapPolarData *) map->data;
-	double r = gog_axis_map_to_view (map->axis_map[1], y);
-	double t = gog_axis_map_to_view (map->axis_map[0], x);
-	
-	*u = data->cx + r * data->rx * cos (t);
-	*v = data->cy + r * data->ry * sin (t);		
-}
-
-/**
- * gog_chart_map_get_polar_parms:
- * @map: a #GogChartMap
- *
- * Convenience function for retrieving data related to polar plot layout.
- *
- * returns: a #GogChartMapPolarData struct.
- **/
-
-GogChartMapPolarData *
-gog_chart_map_get_polar_parms (GogChartMap *map)
-{
-	return (GogChartMapPolarData *) map->data;
-}	
-
-/**
- * gog_chart_map_new:
- * @chart: a #GogChart
- * @area: area allocated to chart
- * @axis0: 1st dimension axis
- * @axis1: 2nd dimension axis
- * @axis2: 3rd dimension axis
- * @fill_area: does chart fill allocated area
- *
- * Creates a new #GogChartMap, used for conversion from data space 
- * to canvas space.
- *
- * returns: a new #GogChart object. 
- **/
-
-GogChartMap *
-gog_chart_map_new (GogChart *chart, GogViewAllocation const *area, 
-		   GogAxis *axis0, GogAxis *axis1, GogAxis *axis2,
-		   gboolean fill_area)
-{
-	GogChartMap *map;
-	GogAxisSet axis_set;
-
-	g_return_val_if_fail (IS_GOG_CHART (chart), NULL);
-
-	map = g_new (GogChartMap, 1);
-
-	g_object_ref (chart);
-	map->chart = chart;
-	map->area = *area;
-	map->data = NULL;
-	map->is_valid = FALSE;
-
-	axis_set = gog_chart_get_axis_set (chart);
-	switch (axis_set & GOG_AXIS_SET_FUNDAMENTAL) {
-		case GOG_AXIS_SET_X:
-			{
-				XMapData *data = g_new (XMapData, 1);
-
-				map->axis_map[0] = gog_axis_map_new (axis0, map->area.x, map->area.w);
-				map->axis_map[1] = map->axis_map[2] = NULL;
-
-				data->b = area->y + area->h;
-				data->a = - area->h;
-
-				map->map_2D_to_view = x_map_2D_to_view;
-				map->data = data;
-
-				map->is_valid = gog_axis_map_is_valid (map->axis_map [0]);
-				break;
-			}
-		case GOG_AXIS_SET_XY:
-		case GOG_AXIS_SET_XY_pseudo_3d:
-		case GOG_AXIS_SET_XY_COLOR:
-			{
-				map->axis_map[0] = gog_axis_map_new (axis0, map->area.x, map->area.w);
-				map->axis_map[1] = gog_axis_map_new (axis1, map->area.y + map->area.h, 
-								     -map->area.h);
-				map->axis_map[2] = NULL;
-
-				map->data = NULL;
-				map->map_2D_to_view = xy_map_2D_to_view;
-
-				map->is_valid = gog_axis_map_is_valid (map->axis_map[0]) &&
-					gog_axis_map_is_valid (map->axis_map[1]);
-				break;
-			}
-		case GOG_AXIS_SET_RADAR:
-			{
-				GogChartMapPolarData *data = g_new (GogChartMapPolarData, 1);
-				double minimum, maximum;
-				double z_rotation = gog_axis_get_circular_rotation (axis0) * M_PI / 180.0;
-				double perimeter;
-	
-				map->axis_map[0] = gog_axis_map_new (axis0, 0.0, 1.0);
-				gog_axis_map_get_bounds (map->axis_map[0], &minimum, &maximum);
-				if (gog_axis_is_discrete (axis0)) {
-					data->th0 = go_rint (minimum);
-					data->th1 = go_rint (maximum);
-					calc_polygon_parameters (area, data, fill_area);
-					gog_axis_map_free (map->axis_map[0]);
-					map->axis_map[0] = gog_axis_map_new (axis0, 
-						- M_PI / 2.0 + z_rotation,
-						2.0 * M_PI * (maximum - minimum) / (maximum - minimum + 1));
-				} else {
-					perimeter = gog_axis_get_polar_perimeter (axis0);
-					minimum = minimum * 2.0 * M_PI / perimeter + z_rotation;
-					maximum = maximum * 2.0 * M_PI / perimeter + z_rotation;
-					data->th0 = minimum;
-					data->th1 = maximum;
-					calc_circle_parameters (area, data, fill_area);
-					gog_axis_map_free (map->axis_map[0]);
-					map->axis_map[0] = gog_axis_map_new (axis0, -minimum,
-									     minimum - maximum);
-				}
-				map->axis_map[1] = gog_axis_map_new (axis1, 0.0, 1.0);
-				map->axis_map[2] = NULL;
-
-				map->data = data;
-				map->map_2D_to_view = polar_map_2D_to_view;
-
-				map->is_valid = gog_axis_map_is_valid (map->axis_map[0]) &&
-					gog_axis_map_is_valid (map->axis_map[1]);
-				break;
-			}
-		case GOG_AXIS_SET_XYZ:
-		case GOG_AXIS_SET_ALL:
-		case GOG_AXIS_SET_NONE:
-		case GOG_AXIS_SET_UNKNOWN:
-		default:
-			g_warning ("[Chart::map_new] not implemented for this axis set (%i)",
-				   axis_set);
-			map->map_2D_to_view = null_map_2D;
-			break;
-	}
-
-	return map;
-}
-
-/**
- * gog_chart_map_2D_to_view:
- * @map: a #GogChartMap
- * @x: data x value
- * @y: data y value
- * @u: placeholder for x converted value
- * @v: placeholder for y converted value
- *
- * Converts a 2D coordinate from data space to canvas space.
- **/
-
-void
-gog_chart_map_2D_to_view (GogChartMap *map, double x, double y, double *u, double *v)
-{
-	(map->map_2D_to_view) (map, x, y, u, v);
-}
-
-/**
- * gog_chart_map_get_axis_map:
- * @map: a #GogChartMap
- * @index: axis index
- *
- * Convenience function which returns one of the associated axis_map.
- *
- * Valid values are in range [0..2].
- *
- * returns: a #GogAxisMap.
- **/
-
-GogAxisMap *
-gog_chart_map_get_axis_map (GogChartMap *map, unsigned int i)
-{
-	g_return_val_if_fail (map != NULL, NULL);
-	g_return_val_if_fail (i < 3, NULL);
-
-	return map->axis_map[i];
-}
-
-/**
- * gog_chart_map_is_valid:
- * @map: a #GogChartMap
- *
- * Tests if @map was correctly initializied, i.e. if all associated axis_map
- * are valid (see gog_axis_map_is_valid() ).
- *
- * given 
- * to gog_chart_map_new. 
- * returns: %TRUE if @map is valid.
- **/
-
-gboolean
-gog_chart_map_is_valid (GogChartMap *map)
-{
-	g_return_val_if_fail (map != NULL, FALSE);
-
-	return map->is_valid;
-}
-
-/**
- * gog_chart_map_free:
- * @map: a #GogChartMap
- *
- * Frees @map object.
- **/
-
-void
-gog_chart_map_free (GogChartMap *map)
-{
-	int i;
-
-	g_return_if_fail (map != NULL);
-
-	for (i = 0; i < 3; i++)
-		if (map->axis_map[i] != NULL)
-			gog_axis_map_free (map->axis_map[i]);
-
-	g_free (map->data);
-	g_object_unref (map->chart);
-	g_free (map);
-}
-
 enum {
 	CHART_PROP_0,
 	CHART_PROP_CARDINALITY_VALID,
@@ -691,9 +353,11 @@ static gboolean bubble_axis_can_add (GogObject const *parent) { return axis_can_
 static void bubble_axis_post_add    (GogObject *parent, GogObject *child)  { axis_post_add   (child, GOG_AXIS_BUBBLE); }
 static gboolean color_axis_can_add (GogObject const *parent) { return axis_can_add (parent, GOG_AXIS_COLOR); }
 static void color_axis_post_add    (GogObject *parent, GogObject *child)  { axis_post_add   (child, GOG_AXIS_COLOR); }
+static gboolean role_3d_box_can_add	(GogObject const *parent) {return FALSE;}
+static gboolean role_3d_box_can_remove	(GogObject const *parent) {return FALSE;}
 
 static GogObjectRole const roles[] = {
-	{ N_("Grid"), "GogGrid",	0,
+	{ N_("Backplane"), "GogGrid",	0,
 	  GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_ROLE,
 	  role_grid_can_add, NULL, NULL, role_grid_post_add, role_grid_pre_remove, NULL, { -1 } },
 	{ N_("X-Axis"), "GogAxis",	1,
@@ -740,7 +404,10 @@ static GogObjectRole const roles[] = {
 	  GOG_POSITION_COMPASS|GOG_POSITION_ANY_MANUAL, 
 	  GOG_POSITION_E|GOG_POSITION_ALIGN_CENTER, 
 	  GOG_OBJECT_NAME_BY_ROLE,
-	  NULL, NULL, NULL, NULL, NULL, NULL, { -1 } }
+	  NULL, NULL, NULL, NULL, NULL, NULL, { -1 } },
+	{ N_("3D-Box"), "Gog3DBox",	0,
+	  GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_ROLE,
+	  role_3d_box_can_add, role_3d_box_can_remove, NULL, NULL, NULL, NULL, { -1 } }
 };
 
 static void
@@ -865,8 +532,9 @@ gog_chart_set_position (GogChart *chart,
  * @chart : #GogChart
  * @plot_area  : #GogViewAllocation
  *
- * Stores plot area in plot_area, in fraction of chart size, and returns
- * TRUE if plot area position is manual.
+ * Stores plot area in plot_area, in fraction of chart size.
+ *
+ * Returns: %TRUE if plot area position is manual.
  **/
 gboolean
 gog_chart_get_plot_area (GogChart *chart, GogViewAllocation *plot_area)
@@ -1100,6 +768,144 @@ gog_chart_get_grid (GogChart const *chart)
 	return GOG_GRID (chart->grid);
 }
 
+gboolean
+gog_chart_is_3d (GogChart const *chart)
+{
+	return chart->axis_set == GOG_AXIS_SET_XYZ;
+}
+
+static void
+gog_chart_3d_process (GogChart *chart, GogViewAllocation const *bbox)
+{
+	/* A XYZ axis set in supposed. If new sets (cylindrical, spherical or
+	other are added, we'll need to change this code */
+	GogViewAllocation tmp = *bbox;
+	GogAxis *axisX, *axisY, *axisZ;
+	GSList *axes;
+	double xmin, xmax, ymin, ymax, zmin, zmax;
+	double o[3], x[3], y[3], z[3], tg, d;
+	Gog3DBox *box;
+	GogObject *obj = gog_object_get_child_by_name (GOG_OBJECT (chart), "3D-Box");
+
+	if (!obj) {
+		obj = g_object_new (GOG_3D_BOX_TYPE, NULL);
+		gog_object_add_by_name (GOG_OBJECT (chart), "3D-Box", obj);
+	}
+	box = GOG_3D_BOX (obj);
+
+	/* Only use the first of the axes. */
+	axes = gog_chart_get_axes (chart, GOG_AXIS_X);
+	axisX = GOG_AXIS (axes->data);
+	g_slist_free (axes);
+	gog_axis_get_bounds (axisX, &xmin, &xmax);
+	axes = gog_chart_get_axes (chart, GOG_AXIS_Y);
+	axisY = GOG_AXIS (axes->data);
+	g_slist_free (axes);
+	gog_axis_get_bounds (axisY, &ymin, &ymax);
+	axes = gog_chart_get_axes (chart, GOG_AXIS_Z);
+	axisZ = GOG_AXIS (axes->data);
+	g_slist_free (axes);
+	gog_axis_get_bounds (axisZ, &zmin, &zmax);
+	/* define the 3d box */
+	/* FIXME: take axes into account */
+	box->dz = tmp.h;
+	if (ymax - ymin > xmax - xmin) {
+		box->dy = tmp.w;
+		box->dx = (xmax - xmin) / (ymax - ymin) * tmp.w;
+	} else {
+		box->dx = tmp.w;
+		box->dy = (ymax - ymin) / (xmax - xmin) * tmp.w;
+	}
+
+	/* now compute the position of each vertex, ignoring the fov */
+	go_matrix3x3_transform (&box->mat, -box->dx, -box->dy, -box->dz, o, o + 1, o + 2);
+	go_matrix3x3_transform (&box->mat, box->dx, -box->dy, -box->dz, x, x + 1, x + 2);
+	go_matrix3x3_transform (&box->mat, -box->dx, box->dy, -box->dz, y, y + 1, y + 2);
+	go_matrix3x3_transform (&box->mat, -box->dx, -box->dy, box->dz, z, z + 1, z + 2);
+	/* for each diagonal, we need to take the vertex closer to the view point */
+	if (o[1] > 0) {
+		o[0] = -o[0];
+		o[1] = -o[1];
+		o[2] = -o[2];
+	}
+	if (x[1] > 0) {
+		x[0] = -x[0];
+		x[1] = -x[1];
+		x[2] = -x[2];
+	}
+	if (y[1] > 0) {
+		y[0] = -y[0];
+		y[1] = -y[1];
+		y[2] = -y[2];
+	}
+	if (z[1] > 0) {
+		z[0] = -z[0];
+		z[1] = -z[1];
+		z[2] = -z[2];
+	}
+	/* if the fov is positive, calculate the position of the viewpoint */
+	if (box->fov > 0.) {
+		tg = tan (box->fov / 2.);
+		box->r = -sqrt (o[0] * o[0] + o[2] * o[2]) / tg + o[1];
+		d = -sqrt (x[0] * x[0] + x[2] * x[2]) / tg + x[1];
+		if (d < box->r)
+			box->r = d;
+		d = -sqrt (y[0] * y[0] + y[2] * y[2]) / tg + y[1];
+		if (d < box->r)
+			box->r = d;
+		d = -sqrt (z[0] *z[0] + z[2] * z[2]) / tg + z[1];
+		if (d < box->r)
+			box->r = d;
+		/* also calculate the reduction factor we need to make things fit in the bbox */
+		xmax = fabs (o[0]) * (1. + o[1] / box->r);
+		zmax = fabs (o[2]) * (1. + o[1] / box->r);
+		d = fabs (x[0]) * (1. + x[1] / box->r);
+		if (d > xmax)
+			xmax = d;
+		d = fabs (x[2]) * (1. + x[1] / box->r);
+		if (d > zmax)
+			zmax = d;
+		d = fabs (y[0]) * (1. + y[1] / box->r);
+		if (d > xmax)
+			xmax = d;
+		d = fabs (y[2]) * (1. + y[1] / box->r);
+		if (d > zmax)
+			zmax = d;
+		d = fabs (z[0]) * (1. + z[1] / box->r);
+		if (d > xmax)
+			xmax = d;
+		d = fabs (z[2]) * (1. + z[1] / box->r);
+		if (d > zmax)
+			zmax = d;
+	} else {
+	    /* calculate the reduction factor we need to make things fit in the bbox */
+		xmax = fabs (o[0]);
+		zmax = fabs (o[2]);
+		d = fabs (x[0]);
+		if (d > xmax)
+			xmax = d;
+		d = fabs (x[2]);
+		if (d > zmax)
+			zmax = d;
+		d = fabs (y[0]);
+		if (d > xmax)
+			xmax = d;
+		d = fabs (y[2]);
+		if (d > zmax)
+			zmax = d;
+		d = fabs (z[0]);
+		if (d > xmax)
+			xmax = d;
+		d = fabs (z[2]);
+		if (d > zmax)
+			zmax = d;
+	}
+	/* use d and tg as x and z ratios, respectively */
+	d = xmax / tmp.w;
+	tg = zmax / tmp.h;
+	box->ratio = (d > tg)? d: tg;
+}
+
 /*********************************************************************/
 
 typedef struct {
@@ -1163,6 +969,10 @@ gog_chart_view_size_allocate (GogView *view, GogViewAllocation const *bbox)
 		if (GOG_POSITION_IS_SPECIAL (child->model->position))
 			gog_view_size_allocate (child, plot_area);
 	}
+
+	/* special treatment for 3d charts */
+	if (gog_chart_is_3d (chart))
+	    gog_chart_3d_process (chart, plot_area);
 }
 
 static void
@@ -1236,24 +1046,38 @@ gog_chart_view_render (GogView *view, GogViewAllocation const *bbox)
 	GSList *ptr;
 	GogView *child_view;
 	gboolean grid_line_rendered = FALSE;
+	GogChart *chart = GOG_CHART (gog_view_get_model (view));
 
 	cview_parent_klass->render (view, bbox);
 
-	/* KLUDGE: render grid lines before axis */
-	for (ptr = view->children ; ptr != NULL ; ptr = ptr->next) {
-		child_view = ptr->data;
-		if (!grid_line_rendered && IS_GOG_AXIS (child_view->model)) {
-			grid_line_render (ptr, bbox);
-			plot_render (view, bbox);
-			grid_line_rendered = TRUE;
+	if (gog_chart_is_3d (chart)) {
+		for (ptr = view->children ; ptr != NULL ; ptr = ptr->next) {
+			child_view = ptr->data;
+			if (!IS_GOG_AXIS (child_view->model) && !IS_GOG_PLOT (child_view->model)) 
+			    gog_view_render	(ptr->data, bbox);
+		}
+		/* now render plot and axes */
+		for (ptr = view->children ; ptr != NULL ; ptr = ptr->next) {
+			child_view = ptr->data;
+			if (IS_GOG_PLOT (child_view->model)) 
+			    gog_view_render	(ptr->data, bbox);
+		}
+	} else {
+		/* KLUDGE: render grid lines before axis */
+		for (ptr = view->children ; ptr != NULL ; ptr = ptr->next) {
+			child_view = ptr->data;
+			if (!grid_line_rendered && IS_GOG_AXIS (child_view->model)) {
+				grid_line_render (ptr, bbox);
+				plot_render (view, bbox);
+				grid_line_rendered = TRUE;
+			}
+			if (IS_GOG_PLOT (child_view->model)) {
+			    if (!GOG_PLOT (child_view->model)->render_before_axes)
+				gog_view_render	(ptr->data, bbox);
+			} else
+				gog_view_render	(ptr->data, bbox);
 		}
-		if (IS_GOG_PLOT (child_view->model)) {
-		    if (!GOG_PLOT (child_view->model)->render_before_axes)
-			gog_view_render	(ptr->data, bbox);
-		} else
-			gog_view_render	(ptr->data, bbox);
 	}
-
 }
 
 static void
