@@ -361,10 +361,10 @@ go_format_get_family (GOFormat const *fmt)
 	case GO_FMT_NUMBER:
 		if (fmt->u.number.is_general)
 			return GO_FORMAT_GENERAL;
-		if (fmt->u.number.has_date)
-			return GO_FORMAT_DATE;
 		if (fmt->u.number.has_time)
 			return GO_FORMAT_TIME;
+		if (fmt->u.number.has_date)
+			return GO_FORMAT_DATE;
 		if (fmt->u.number.fraction)
 			return GO_FORMAT_FRACTION;
 		if (fmt->u.number.E_format)
@@ -1541,11 +1541,6 @@ go_format_parse_number_E (GOFormatParseState *pstate)
 	if (tno_exp_start >= tno_end)
 		return NULL;
 	switch (GET_TOKEN (pstate->tno_E + 1).token) {
-	case '-':
-		use_markup = FALSE;
-		simplify_mantissa = FALSE;
-		pstate->forced_exponent_sign = FALSE;
-		break;
 	case '+':
 		use_markup = FALSE;
 		simplify_mantissa = FALSE;
@@ -3841,11 +3836,10 @@ go_format_parse_markup (char *str)
 #ifdef DEFINE_COMMON
 /**
  * go_format_new_from_XL :
+ * @str: XL descriptor in UTF-8 encoding.
  *
- * Looks up and potentially creates a GOFormat from the supplied string in
- * XL format.
- *
- * @descriptor_string: XL descriptor in UTF-8 encoding.
+ * Returns: Looks up and potentially creates a GOFormat from the supplied
+ * 	string in XL format.
  **/
 GOFormat *
 go_format_new_from_XL (char const *str)
@@ -3890,11 +3884,13 @@ go_format_new_from_XL (char const *str)
 /**
  * go_format_new_markup :
  * @markup : #PangoAttrList
- * @add_ref :
+ * @add_ref : boolean
  *
- * Create a MARKUP format.  If @add_ref is FALSE absorb the reference to
- * @markup, otherwise add a reference.
- */
+ * If @add_ref is FALSE absorb the reference to @markup, otherwise add a
+ * reference.
+ *
+ * Returns: A new format.
+ **/
 GOFormat *
 go_format_new_markup (PangoAttrList *markup, gboolean add_ref)
 {
@@ -3949,6 +3945,7 @@ go_format_eq (GOFormat const *a, GOFormat const *b)
  *
  * Adds a reference to a GOFormat.
  *
+ * Returns: @gf
  **/
 GOFormat *
 go_format_ref (GOFormat *gf)
