@@ -35,6 +35,42 @@
 #include <glib/gi18n-lib.h>
 #include <gsf/gsf-impl-utils.h>
 
+/******************************************************************************/
+
+typedef GogSeriesElement GogBarColSeriesElement;
+typedef GogSeriesElementClass GogBarColSeriesElementClass;
+
+#define GOG_BARCOL_SERIES_ELEMENT_TYPE	(gog_barcol_series_element_get_type ())
+#define GOG_BARCOL_SERIES_ELEMENT(o)	(G_TYPE_CHECK_INSTANCE_CAST ((o), GOG_BARCOL_SERIES_ELEMENT_TYPE, GogBarColSeriesElement))
+#define GOG_IS_BARCOL_SERIES_ELEMENT(o)	(G_TYPE_CHECK_INSTANCE_TYPE ((o), GOG_BARCOL_SERIES_ELEMENT_TYPE))
+GType gog_barcol_series_element_get_type (void);
+
+GSF_DYNAMIC_CLASS (GogBarColSeriesElement, gog_barcol_series_element,
+	NULL, NULL,
+	GOG_SERIES_ELEMENT_TYPE)
+
+/******************************************************************************/
+
+typedef GogSeries1_5d GogBarColSeries;
+typedef GogSeries1_5dClass GogBarColSeriesClass;
+
+#define GOG_BARCOL_SERIES_TYPE	(gog_barcol_series_eget_type ())
+#define GOG_BARCOL_SERIES(o)	(G_TYPE_CHECK_INSTANCE_CAST ((o), GOG_BARCOL_SERIES_TYPE, GogBarColSeries))
+#define GOG_IS_BARCOL_SERIEST(o)	(G_TYPE_CHECK_INSTANCE_TYPE ((o), GOG_BARCOL_SERIES_TYPE))
+GType gog_barcol_series_get_type (void);
+
+static void
+gog_barcol_series_class_init (GogSeriesClass *series_klass)
+{
+	series_klass->series_element_type = GOG_BARCOL_SERIES_ELEMENT_TYPE;
+}
+
+GSF_DYNAMIC_CLASS (GogBarColSeries, gog_barcol_series,
+	gog_barcol_series_class_init, NULL,
+	GOG_SERIES1_5D_TYPE)
+
+/******************************************************************************/
+
 enum {
 	BARCOL_PROP_0,
 	BARCOL_PROP_GAP_PERCENTAGE,
@@ -223,6 +259,7 @@ gog_barcol_plot_class_init (GogPlot1_5dClass *gog_plot_1_5d_klass)
 	gog_object_klass->view_type	= gog_barcol_view_get_type ();
 
 	plot_klass->desc.series.style_fields	= GOG_STYLE_OUTLINE | GOG_STYLE_FILL;
+	plot_klass->series_type = gog_barcol_series_get_type ();
 	plot_klass->axis_get_bounds   		= gog_barcol_axis_get_bounds;
 
 	gog_plot_1_5d_klass->swap_x_and_y = gog_barcol_swap_x_and_y;
@@ -338,11 +375,13 @@ gog_barcol_view_render (GogView *view, GogViewAllocation const *bbox)
 	ErrorBarData **error_data;
 	GogErrorBar **errors;
 	GogSeriesLines **lines;
-	ArtVpath **paths;
+	GOPath **paths;
 	GSList *ptr;
 	unsigned *lengths;
 	double plus, minus;
 	GogObjectRole const *role = NULL;
+	GogSeriesElement *gse;
+	GList const *overrides;
 
 	if (num_elements <= 0 || num_series <= 0)
 		return;
@@ -368,7 +407,7 @@ gog_barcol_view_render (GogView *view, GogViewAllocation const *bbox)
 	errors = g_alloca (num_series * sizeof (GogErrorBar *));
 	error_data = g_alloca (num_series * sizeof (ErrorBarData *));
 	lines = g_alloca (num_series * sizeof (GogSeriesLines *));
-	paths = g_alloca (num_series * sizeof (ArtVpath *));
+	paths = g_alloca (num_series * sizeof (GOPath *));
 	
 	i = 0;
 	for (ptr = gog_1_5d_model->base.series ; ptr != NULL ; ptr = ptr->next) {
@@ -391,12 +430,7 @@ gog_barcol_view_render (GogView *view, GogViewAllocation const *bbox)
 							GOG_OBJECT (series), "Series lines");
 			lines[i] = GOG_SERIES_LINES (
 					gog_object_get_child_by_role (GOG_OBJECT (series), role));
-			paths[i] = g_new (ArtVpath, lengths[i] * 2 - 1);
-			for (j = 0; j < lengths[i] - 1; j++) {
-				paths[i][j * 2].code = ART_MOVETO;
-				paths[i][j * 2 + 1].code = ART_LINETO;
-			}
-			paths[i][j * 2].code = ART_END;
+			paths[i] = go_path_new ();
 		} else
 			lines[i] = NULL;
 		i++;
@@ -429,8 +463,9 @@ gog_barcol_view_render (GogView *view, GogViewAllocation const *bbox)
 		}
 
 		pos_base = neg_base = 0.0;
+		ptr = gog_1_5d_model->base.series;
 		for (j = 0 ; j < num_series ; j++) {
-			
+			overrides = gog_series_get_overrides (GOG_SERIES (ptr->data));
 			work.y = (double) j * col_step + (double) i - offset + 1.0;
 			
 			if (i >= lengths[j])
@@ -456,7 +491,16 @@ gog_barcol_view_render (GogView *view, GogViewAllocation const *bbox)
 					neg_base += tmp;
 			}
 
-			gog_renderer_push_style (view->renderer, styles[j]);
+			gse = NULL;
+			if ((overrides != NULL) &&
+				(GOG_SERIES_ELEMENT (overrides->data)->index == i)) {
+					gse = GOG_SERIES_ELEMENT (overrides->data);
+					overrides = overrides->next;
+					gog_renderer_push_style (view->renderer,
+						gog_styled_object_get_style (
+							GOG_STYLED_OBJECT (gse)));
+			} else
+				gog_renderer_push_style (view->renderer, styles[j]);
 			barcol_draw_rect (rend, is_vertical, x_map, y_map, &work);
 			gog_renderer_pop_style (view->renderer);
 			
@@ -476,20 +520,25 @@ gog_barcol_view_render (GogView *view, GogViewAllocation const *bbox)
 				x = tmp > 0 ? work.x + work.w: work.x;
 				if (is_vertical) {
 					if (i > 0) {
-						paths[j][i * 2 - 1].x = gog_axis_map_to_view (x_map, work.y);
-						paths[j][i * 2 - 1].y = gog_axis_map_to_view (y_map, x);
+						go_path_line_to (paths[j],
+								 gog_axis_map_to_view (x_map, work.y),
+								 gog_axis_map_to_view (y_map, x));
 					}
-					paths[j][i * 2].x = gog_axis_map_to_view (x_map, work.y + work.h);
-					paths[j][i * 2].y = gog_axis_map_to_view (y_map, x);
+					go_path_move_to (paths[j],
+							 gog_axis_map_to_view (x_map, work.y + work.h),
+							 gog_axis_map_to_view (y_map, x));
 				} else {
 					if (i > 0) {
-						paths[j][i * 2 - 1].x = gog_axis_map_to_view (x_map, x);
-						paths[j][i * 2 - 1].y = gog_axis_map_to_view (y_map, work.y);
+						go_path_line_to (paths[j],
+								 gog_axis_map_to_view (x_map, x),
+								 gog_axis_map_to_view (y_map, work.y));
 					}
-					paths[j][i * 2].x = gog_axis_map_to_view (x_map, x);
-					paths[j][i * 2].y = gog_axis_map_to_view (y_map, work.y + work.h);
+					go_path_move_to (paths[j],
+							 gog_axis_map_to_view (x_map, x),
+							 gog_axis_map_to_view (y_map, work.y + work.h));
 				}
 			}
+			ptr = ptr->next;
 		}
 	}
 	/*Now draw error bars and clean*/
@@ -507,8 +556,8 @@ gog_barcol_view_render (GogView *view, GogViewAllocation const *bbox)
 	/* Draw series lines if any */
 	for (i = 0; i < num_series; i++)
 		if (lines[i] != NULL) {
-			gog_series_lines_render (lines[i], view->renderer, bbox, paths[i], FALSE);
-			g_free (paths[i]);
+			gog_series_lines_stroke (lines[i], view->renderer, bbox, paths[i], FALSE);
+			go_path_free (paths[i]);
 		}
 
 	gog_chart_map_free (chart_map);
