@@ -38,6 +38,11 @@
 #include <goffice/utils/go-line.h>
 #include <goffice/app/module-plugin-defs.h>
 
+#ifdef GOFFICE_WITH_GTK
+#include <goffice/gtk/goffice-gtk.h>
+#include <gtk/gtk.h>
+#endif
+
 #include <glib/gi18n-lib.h>
 #include <gsf/gsf-impl-utils.h>
 #include <math.h>
@@ -523,7 +528,7 @@ gog_bubble_plot_class_init (GogPlotClass *plot_klass)
 		g_param_spec_float ("bubble-scale", 
 			_("Bubble scale"),
 			_("Fraction of default radius used for display"),
-			0., 2., 1.,
+			0., 3., 1.,
 			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 	{
 		static GogSeriesDimDesc dimensions[] = {
@@ -549,7 +554,7 @@ gog_bubble_plot_class_init (GogPlotClass *plot_klass)
 	}
 }
 
-#define BUBBLE_MAX_RADIUS_RATIO 8.
+#define BUBBLE_MAX_RADIUS_RATIO 12.	/* 4 * MAX(bubble-scale) */
 static void
 gog_bubble_plot_adjust_bounds (Gog2DPlot *model, double *x_min, double *x_max, double *y_min, double *y_max)
 {
@@ -660,15 +665,12 @@ gog_xy_color_plot_type_name (G_GNUC_UNUSED GogObject const *item)
 }
 
 #ifdef GOFFICE_WITH_GTK
-#include <goffice/gtk/goffice-gtk.h>
-#include <gtk/gtktogglebutton.h>
-#include <gtk/gtkcombobox.h>
-
 static void
 hide_outliers_toggled_cb (GtkToggleButton *btn, GObject *obj)
 {
 	g_object_set (obj, "hide-outliers", gtk_toggle_button_get_active (btn), NULL);
 }
+#endif
 
 static void 
 gog_xy_color_plot_populate_editor (GogObject *obj,
@@ -676,7 +678,7 @@ gog_xy_color_plot_populate_editor (GogObject *obj,
 				   GogDataAllocator *dalloc,
 				   GOCmdContext *cc)
 {
-	GtkWidget *w;
+#ifdef GOFFICE_WITH_GTK
 	GladeXML *gui;
 	char const *dir;
 	char *path;
@@ -687,7 +689,7 @@ gog_xy_color_plot_populate_editor (GogObject *obj,
 	g_free (path);
 
 	if (gui != NULL) {
-		w = glade_xml_get_widget (gui, "hide-outliers");
+		GtkWidget *w = glade_xml_get_widget (gui, "hide-outliers");
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
 				(GOG_XY_COLOR_PLOT (obj))->hide_outliers);
 		g_signal_connect (G_OBJECT (w),
@@ -697,9 +699,9 @@ gog_xy_color_plot_populate_editor (GogObject *obj,
 		gog_editor_add_page (editor, w, _("Properties"));
 	}
 
+#endif
 	(GOG_OBJECT_CLASS(map_parent_klass)->populate_editor) (obj, editor, dalloc, cc);
 }
-#endif
 
 static void
 gog_xy_color_plot_set_property (GObject *obj, guint param_id,
@@ -829,19 +831,6 @@ GSF_DYNAMIC_CLASS (GogXYColorPlot, gog_xy_color_plot,
 typedef GogPlotView		GogXYView;
 typedef GogPlotViewClass	GogXYViewClass;
 
-#define MAX_ARC_SEGMENTS 64
-
-static void
-bubble_draw_circle (GogView *view, double x, double y, double radius)
-{
-	GOPath *path;
-
-	path = go_path_new ();
-	go_path_pie_wedge (path, x, y, radius, radius, 0, 2 * M_PI);
-	gog_renderer_draw_shape (view->renderer, path);
-	go_path_free (path);
-}
-
 static GOColor
 get_map_color (double z, gboolean hide_outliers)
 {
@@ -893,6 +882,9 @@ gog_xy_view_render (GogView *view, GogViewAllocation const *bbox)
 	MarkerData **markers;
 	unsigned *num_markers;
 
+	GogSeriesElement *gse;
+	GList const *overrides;
+
 	for (num_series = 0, ptr = model->base.series ; ptr != NULL ; ptr = ptr->next, num_series++);
 	if (num_series < 1)
 		return;
@@ -1084,14 +1076,18 @@ gog_xy_view_render (GogView *view, GogViewAllocation const *bbox)
 		y_margin_min = view->allocation.y - margin;
 		y_margin_max = view->allocation.y + view->allocation.h + margin;
 
+		overrides = gog_series_get_overrides (GOG_SERIES (series));
 		k = 0;
 		for (i = 1 ; i <= n ; i++) {
 			x = x_vals ? *x_vals++ : i;
 			y = *y_vals++;
+			if (is_map || GOG_IS_BUBBLE_PLOT(model)) {
+				z = *z_vals++;
+				if (isnan (z) || !go_finite (z))
+					continue;
+			}
 			if (isnan (y) || isnan (x))
 				continue;
-			if (is_map && isnan (z = *z_vals++))
-				continue;
 			/* We are checking with go_finite here because isinf
 			   if not available everywhere.  Note, that NANs
 			   have been ruled out.  */
@@ -1102,25 +1098,33 @@ gog_xy_view_render (GogView *view, GogViewAllocation const *bbox)
 			x_canvas = gog_axis_map_to_view (x_map, x);
 			y_canvas = gog_axis_map_to_view (y_map, y);
 			if (GOG_IS_BUBBLE_PLOT(model)) {
-				z = *z_vals++;
-				if (!go_finite (z)) continue;
 				if (z < 0) {
 					if (GOG_BUBBLE_PLOT(model)->show_negatives) {
 						gog_renderer_push_style (view->renderer, neg_style);
-						bubble_draw_circle (view, x_canvas, y_canvas, 
+						gog_renderer_draw_circle (view->renderer, x_canvas, y_canvas, 
 								    ((size_as_area) ? 
 								     sqrt (- z / zmax) :
 								    - z / zmax) * rmax);
 						gog_renderer_pop_style (view->renderer);
 					} else continue;
 				} else {
-					if (model->base.vary_style_by_element)
+					gse = NULL;
+					if ((overrides != NULL) &&
+					   	(GOG_SERIES_ELEMENT (overrides->data)->index == i - 1)) {
+							gse = GOG_SERIES_ELEMENT (overrides->data);
+							overrides = overrides->next;
+							gog_renderer_push_style (view->renderer,
+								gog_styled_object_get_style (
+									GOG_STYLED_OBJECT (gse)));
+					} else if (model->base.vary_style_by_element)
 						gog_theme_fillin_style (theme, style, GOG_OBJECT (series),
 									model->base.index_num + i - 1, FALSE);
-					bubble_draw_circle (view, x_canvas, y_canvas, 
+					gog_renderer_draw_circle (view->renderer, x_canvas, y_canvas, 
 							    ((size_as_area) ?
 							    sqrt (z / zmax) :
 							    z / zmax) * rmax);
+					if (gse)
+						gog_renderer_pop_style (view->renderer);
 				}
 			}
 
@@ -1176,11 +1180,21 @@ gog_xy_view_render (GogView *view, GogViewAllocation const *bbox)
 
 	if (!GOG_IS_BUBBLE_PLOT (model))
 		for (j = 0, ptr = model->base.series ; ptr != NULL ; ptr = ptr->next, j++) {
+			overrides = gog_series_get_overrides (GOG_SERIES (series));
 				if (markers[j] != NULL) {
 					series = ptr->data;
+					overrides = gog_series_get_overrides (GOG_SERIES (series));
 					style = GOG_STYLED_OBJECT (series)->style;
 					gog_renderer_push_style (view->renderer, style);
 					for (k = 0; k < num_markers[j]; k++) {
+						gse = NULL;
+						if ((overrides != NULL) &&
+							(GOG_SERIES_ELEMENT (overrides->data)->index == k)) {
+								gse = GOG_SERIES_ELEMENT (overrides->data);
+								overrides = overrides->next;
+								style = gog_styled_object_get_style (GOG_STYLED_OBJECT (gse));
+								gog_renderer_push_style (view->renderer, style);
+						}
 						if (is_map) {
 							go_marker_set_outline_color
 								(style->marker.mark,markers[j][k].color);
@@ -1193,6 +1207,10 @@ gog_xy_view_render (GogView *view, GogViewAllocation const *bbox)
 									  markers[j][k].y);
 						if (is_map)
 							gog_renderer_pop_style (view->renderer);
+						if (gse) {
+							gog_renderer_pop_style (view->renderer);
+							style = GOG_STYLED_OBJECT (series)->style;
+						}
 					}
 					gog_renderer_pop_style (view->renderer);
 					g_free (markers[j]);
@@ -1321,6 +1339,43 @@ vert_drop_lines_pre_remove (GogObject *parent, GogObject *child)
 
 /****************************************************************************/
 
+typedef GogSeriesElement GogXYSeriesElement;
+typedef GogSeriesElementClass GogXYSeriesElementClass;
+#define GOG_XY_SERIES_ELEMENT_TYPE	(gog_xy_series_element_get_type ())
+#define GOG_XY_SERIES_ELEMENT(o)	(G_TYPE_CHECK_INSTANCE_CAST ((o), GOG_XY_SERIES_ELEMENT_TYPE, GogXYSeriesElement))
+#define IS_GOG_XY_SERIES_ELEMENT(o)	(G_TYPE_CHECK_INSTANCE_TYPE ((o), GOG_XY_SERIES_ELEMENT_TYPE))
+GType gog_xy_series_element_get_type (void);
+
+static void
+gog_xy_series_element_init_style (GogStyledObject *gso, GogStyle *style)
+{
+	GogSeries const *series = GOG_SERIES (GOG_OBJECT (gso)->parent);
+	GogStyle *parent_style;
+
+	g_return_if_fail (series != NULL);
+
+	parent_style = gog_styled_object_get_style (GOG_STYLED_OBJECT (series));
+	if (parent_style->interesting_fields & GOG_STYLE_MARKER)
+		style->interesting_fields = parent_style->interesting_fields & (GOG_STYLE_MARKER | GOG_STYLE_MARKER_NO_COLOR);
+	else
+		style->interesting_fields = parent_style->interesting_fields;
+	gog_theme_fillin_style (gog_object_get_theme (GOG_OBJECT (gso)),
+		style, GOG_OBJECT (gso), GOG_SERIES_ELEMENT (gso)->index, FALSE);
+}
+
+static void
+gog_xy_series_element_class_init (GogXYSeriesElementClass *klass)
+{
+	GogStyledObjectClass *style_klass = (GogStyledObjectClass *) klass;
+	style_klass->init_style	    	= gog_xy_series_element_init_style;
+}
+
+GSF_DYNAMIC_CLASS (GogXYSeriesElement, gog_xy_series_element,
+	gog_xy_series_element_class_init, NULL,
+	GOG_SERIES_ELEMENT_TYPE)
+
+/****************************************************************************/
+
 typedef GogSeriesClass GogXYSeriesClass;
 
 enum {
@@ -1597,6 +1652,7 @@ gog_xy_series_class_init (GogStyledObjectClass *gso_klass)
 
 	series_klass->has_interpolation = TRUE;
 	series_klass->has_fill_type	= TRUE;
+	series_klass->series_element_type = GOG_XY_SERIES_ELEMENT_TYPE;
 
 	gog_object_register_roles (gog_klass, roles, G_N_ELEMENTS (roles));
 
@@ -1637,6 +1693,7 @@ go_plugin_init (GOPlugin *plugin, GOCmdContext *cc)
 	gog_xy_view_register_type (module);
 	gog_xy_series_view_register_type (module);
 	gog_xy_series_register_type (module);
+	gog_xy_series_element_register_type (module);
 }
 
 G_MODULE_EXPORT void
